@@ -59,14 +59,14 @@
 {
   GdkWindow *window = [[self contentView] gdkWindow];
 
-  _gdk_quartz_update_focus_window (window, TRUE);
+  _gdk_quartz_events_update_focus_window (window, TRUE);
 }
 
--(void)windowDidResignMain:(NSNotification *)aNotification
+-(void)windowDidResignKey:(NSNotification *)aNotification
 {
   GdkWindow *window = [[self contentView] gdkWindow];
 
-  _gdk_quartz_update_focus_window (window, FALSE);
+  _gdk_quartz_events_update_focus_window (window, FALSE);
 }
 
 -(void)windowDidMove:(NSNotification *)aNotification
@@ -78,7 +78,7 @@
   GdkEvent *event;
 
   private->x = content_rect.origin.x;
-  private->y = _gdk_quartz_get_inverted_screen_y (content_rect.origin.y) - impl->height;
+  private->y = _gdk_quartz_window_get_inverted_screen_y (content_rect.origin.y) - impl->height;
 
   /* Synthesize a configure event */
   event = gdk_event_new (GDK_CONFIGURE);
@@ -176,6 +176,15 @@
   GdkWindowObject *private = (GdkWindowObject *)window;
   GdkWindowImplQuartz *impl = GDK_WINDOW_IMPL_QUARTZ (private->impl);
 
+  if (!private->accept_focus)
+    return NO;
+
+  /* Popup windows should not be able to get focused in the window
+   * manager sense, it's only handled through grabs.
+   */
+  if (private->window_type == GDK_WINDOW_TEMP)
+    return NO;
+
   switch (impl->type_hint)
     {
     case GDK_WINDOW_TYPE_HINT_NORMAL:
@@ -290,7 +299,7 @@ update_context_from_dragging_info (id <NSDraggingInfo> sender)
   event.dnd.context = current_context;
   event.dnd.time = GDK_CURRENT_TIME;
   event.dnd.x_root = screen_point.x;
-  event.dnd.y_root = _gdk_quartz_get_inverted_screen_y (screen_point.y);
+  event.dnd.y_root = _gdk_quartz_window_get_inverted_screen_y (screen_point.y);
 
   (*_gdk_event_func) (&event, _gdk_event_data);
 
@@ -313,7 +322,7 @@ update_context_from_dragging_info (id <NSDraggingInfo> sender)
   event.dnd.context = current_context;
   event.dnd.time = GDK_CURRENT_TIME;
   event.dnd.x_root = screen_point.x;
-  event.dnd.y_root = _gdk_quartz_get_inverted_screen_y (screen_point.y);
+  event.dnd.y_root = _gdk_quartz_window_get_inverted_screen_y (screen_point.y);
 
   (*_gdk_event_func) (&event, _gdk_event_data);
 
