@@ -120,7 +120,7 @@ gtk_container_get_type (void)
 
   if (!container_type)
     {
-      static const GTypeInfo container_info =
+      const GTypeInfo container_info =
       {
 	sizeof (GtkContainerClass),
 	(GBaseInitFunc) gtk_container_base_class_init,
@@ -135,7 +135,7 @@ gtk_container_get_type (void)
       };
 
       container_type =
-	g_type_register_static (GTK_TYPE_WIDGET, "GtkContainer", 
+	g_type_register_static (GTK_TYPE_WIDGET, I_("GtkContainer"), 
 				&container_info, G_TYPE_FLAG_ABSTRACT);
     }
 
@@ -224,7 +224,7 @@ gtk_container_class_init (GtkContainerClass *class)
                                                       GTK_TYPE_WIDGET,
 						      GTK_PARAM_WRITABLE));
   container_signals[ADD] =
-    g_signal_new ("add",
+    g_signal_new (I_("add"),
 		  G_OBJECT_CLASS_TYPE (object_class),
 		  G_SIGNAL_RUN_FIRST,
 		  G_STRUCT_OFFSET (GtkContainerClass, add),
@@ -233,7 +233,7 @@ gtk_container_class_init (GtkContainerClass *class)
 		  G_TYPE_NONE, 1,
 		  GTK_TYPE_WIDGET);
   container_signals[REMOVE] =
-    g_signal_new ("remove",
+    g_signal_new (I_("remove"),
 		  G_OBJECT_CLASS_TYPE (object_class),
 		  G_SIGNAL_RUN_FIRST,
 		  G_STRUCT_OFFSET (GtkContainerClass, remove),
@@ -242,7 +242,7 @@ gtk_container_class_init (GtkContainerClass *class)
 		  G_TYPE_NONE, 1,
 		  GTK_TYPE_WIDGET);
   container_signals[CHECK_RESIZE] =
-    g_signal_new ("check_resize",
+    g_signal_new (I_("check_resize"),
 		  G_OBJECT_CLASS_TYPE (object_class),
 		  G_SIGNAL_RUN_LAST,
 		  G_STRUCT_OFFSET (GtkContainerClass, check_resize),
@@ -250,7 +250,7 @@ gtk_container_class_init (GtkContainerClass *class)
 		  _gtk_marshal_VOID__VOID,
 		  G_TYPE_NONE, 0);
   container_signals[SET_FOCUS_CHILD] =
-    g_signal_new ("set-focus-child",
+    g_signal_new (I_("set-focus-child"),
 		  G_OBJECT_CLASS_TYPE (object_class),
 		  G_SIGNAL_RUN_FIRST,
 		  G_STRUCT_OFFSET (GtkContainerClass, set_focus_child),
@@ -262,7 +262,7 @@ gtk_container_class_init (GtkContainerClass *class)
 
 /**
  * gtk_container_child_type: 
- * @container: a #GtkContainer.
+ * @container: a #GtkContainer
  *
  * Returns the type of the children supported by the container.
  *
@@ -487,7 +487,7 @@ gtk_container_child_get_property (GtkContainer *container,
  * @child: a widget which is a child of @container
  * @first_property_name: the name of the first property to set
  * @var_args: a %NULL-terminated list of property names and values, starting
- *           with @first_prop_name.
+ *           with @first_prop_name
  * 
  * Sets one or more child properties for @child and @container.
  **/
@@ -497,7 +497,6 @@ gtk_container_child_set_valist (GtkContainer *container,
 				const gchar  *first_property_name,
 				va_list       var_args)
 {
-  GObject *object;
   GObjectNotifyQueue *nqueue;
   const gchar *name;
 
@@ -508,7 +507,6 @@ gtk_container_child_set_valist (GtkContainer *container,
   g_object_ref (container);
   g_object_ref (child);
 
-  object = G_OBJECT (container);
   nqueue = g_object_notify_queue_freeze (G_OBJECT (child), _gtk_widget_child_property_notify_context);
   name = first_property_name;
   while (name)
@@ -572,7 +570,6 @@ gtk_container_child_set_property (GtkContainer *container,
 				  const gchar  *property_name,
 				  const GValue *value)
 {
-  GObject *object;
   GObjectNotifyQueue *nqueue;
   GParamSpec *pspec;
 
@@ -585,7 +582,6 @@ gtk_container_child_set_property (GtkContainer *container,
   g_object_ref (container);
   g_object_ref (child);
 
-  object = G_OBJECT (container);
   nqueue = g_object_notify_queue_freeze (G_OBJECT (child), _gtk_widget_child_property_notify_context);
   pspec = g_param_spec_pool_lookup (_gtk_widget_child_property_pool, property_name,
 				    G_OBJECT_TYPE (container), TRUE);
@@ -614,7 +610,7 @@ gtk_container_child_set_property (GtkContainer *container,
  * @widget: a widget to be placed inside @container 
  * @first_prop_name: the name of the first child property to set 
  * @Varargs: a %NULL-terminated list of property names and values, starting
- *           with @first_prop_name.
+ *           with @first_prop_name
  * 
  * Adds @widget to @container, setting child properties at the same time.
  * See gtk_container_add() and gtk_container_child_set() for more details.
@@ -654,7 +650,7 @@ gtk_container_add_with_properties (GtkContainer *container,
  * @child: a widget which is a child of @container
  * @first_prop_name: the name of the first property to set
  * @Varargs: a %NULL-terminated list of property names and values, starting
- *           with @first_prop_name.
+ *           with @first_prop_name
  * 
  * Sets one or more child properties for @child and @container.
  **/
@@ -681,7 +677,7 @@ gtk_container_child_set (GtkContainer      *container,
  * @child: a widget which is a child of @container
  * @first_prop_name: the name of the first property to get
  * @Varargs: a %NULL-terminated list of property names and #GValue*, 
- *           starting with @first_prop_name.
+ *           starting with @first_prop_name
  * 
  * Gets the values of one or more child properties for @child and @container.
  **/
@@ -765,8 +761,8 @@ gtk_container_class_find_child_property (GObjectClass *cclass,
  * gtk_container_class_list_child_properties:
  * @cclass: a #GtkContainerClass
  * @n_properties: location to return the number of child properties found
- * @returns: a newly allocated array of #GParamSpec*. The array must be 
- *           freed with g_free().
+ * @returns: a newly allocated %NULL-terminated array of #GParamSpec*. 
+ *           The array must be freed with g_free().
  *
  * Returns all child properties of a container class.
  */
@@ -882,8 +878,8 @@ gtk_container_get_property (GObject         *object,
 /**
  * gtk_container_set_border_width:
  * @container: a #GtkContainer
- * @border_width: amount of blank space to leave <emphasis>outside</emphasis> the container.
- *   Valid values are in the range 0-65535 pixels.
+ * @border_width: amount of blank space to leave <emphasis>outside</emphasis> 
+ *   the container. Valid values are in the range 0-65535 pixels.
  *
  * Sets the border width of the container.
  *
@@ -1007,8 +1003,8 @@ _gtk_container_dequeue_resize_handler (GtkContainer *container)
 
 /**
  * gtk_container_set_resize_mode:
- * @container: a #GtkContainer.
- * @resize_mode: the new resize mode.
+ * @container: a #GtkContainer
+ * @resize_mode: the new resize mode
  * 
  * Sets the resize mode for the container.
  *
@@ -1057,8 +1053,8 @@ gtk_container_get_resize_mode (GtkContainer *container)
 
 /**
  * gtk_container_set_reallocate_redraws:
- * @container: a #GtkContainer.
- * @needs_redraws: the new value for the container's @reallocate_redraws flag.
+ * @container: a #GtkContainer
+ * @needs_redraws: the new value for the container's @reallocate_redraws flag
  *
  * Sets the @reallocate_redraws flag of the container to the given value.
  * 
@@ -1092,8 +1088,6 @@ gtk_container_get_resize_container (GtkContainer *container)
 static gboolean
 gtk_container_idle_sizer (gpointer data)
 {
-  GDK_THREADS_ENTER ();
-
   /* we may be invoked with a container_resize_queue of NULL, because
    * queue_resize could have been adding an extra idle function while
    * the queue still got processed. we better just ignore such case
@@ -1116,8 +1110,6 @@ gtk_container_idle_sizer (gpointer data)
 
   gdk_window_process_all_updates ();
 
-  GDK_THREADS_LEAVE ();
-
   return FALSE;
 }
 
@@ -1155,7 +1147,7 @@ _gtk_container_queue_resize (GtkContainer *container)
 		{
 		  GTK_PRIVATE_SET_FLAG (resize_container, GTK_RESIZE_PENDING);
 		  if (container_resize_queue == NULL)
-		    g_idle_add_full (GTK_PRIORITY_RESIZE,
+		    gdk_threads_add_idle_full (GTK_PRIORITY_RESIZE,
 				     gtk_container_idle_sizer,
 				     NULL, NULL);
 		  container_resize_queue = g_slist_prepend (container_resize_queue, resize_container);
@@ -1271,7 +1263,7 @@ gtk_container_forall (GtkContainer *container,
  * @callback: a callback
  * @callback_data: callback user data
  * 
- * Invokes @callback on each non-internal child of @container.  See
+ * Invokes @callback on each non-internal child of @container. See
  * gtk_container_forall() for details on what constitutes an
  * "internal" child.  Most applications should use
  * gtk_container_foreach(), rather than gtk_container_forall().
@@ -1362,7 +1354,7 @@ gtk_container_set_focus_child (GtkContainer *container,
 
 /**
  * gtk_container_get_children:
- * @container: a #GtkContainer.
+ * @container: a #GtkContainer
  * 
  * Returns the container's non-internal children. See
  * gtk_container_forall() for details on what constitutes an "internal" child. 
@@ -1958,13 +1950,13 @@ gtk_container_focus_sort_left_right (GtkContainer     *container,
  * gtk_container_focus_sort:
  * @container: a #GtkContainer
  * @children:  a list of descendents of @container (they don't
- *             have to be direct children.
+ *             have to be direct children)
  * @direction: focus direction
  * @old_focus: widget to use for the starting position, or %NULL
  *             to determine this automatically.
- *             [ Note, this argument isn't used for GTK_DIR_TAB_*,
- *               which is the only @direction we use currently,
- *               so perhaps this argument should be removed ]
+ *             (Note, this argument isn't used for GTK_DIR_TAB_*,
+ *              which is the only @direction we use currently,
+ *              so perhaps this argument should be removed)
  * 
  * Sorts @children in the correct order for focusing with
  * direction type @direction.
@@ -2075,14 +2067,14 @@ chain_widget_destroyed (GtkWidget *widget,
                                         user_data);
   
   g_object_set_data (G_OBJECT (container),
-                     "gtk-container-focus-chain",
+                     I_("gtk-container-focus-chain"),
                      chain);  
 }
 
 /**
  * gtk_container_set_focus_chain: 
- * @container: a #GtkContainer.
- * @focusable_widgets: the new focus chain.
+ * @container: a #GtkContainer
+ * @focusable_widgets: the new focus chain
  *
  * Sets a focus chain, overriding the one computed automatically by GTK+.
  * 
@@ -2132,7 +2124,7 @@ gtk_container_set_focus_chain (GtkContainer *container,
   chain = g_list_reverse (chain);
   
   g_object_set_data (G_OBJECT (container),
-                     "gtk-container-focus-chain",
+                     I_("gtk-container-focus-chain"),
                      chain);
 }
 
@@ -2173,7 +2165,7 @@ gtk_container_get_focus_chain (GtkContainer *container,
 
 /**
  * gtk_container_unset_focus_chain:
- * @container: a #GtkContainer.
+ * @container: a #GtkContainer
  * 
  * Removes a focus chain explicitly set with gtk_container_set_focus_chain().
  **/
@@ -2191,7 +2183,8 @@ gtk_container_unset_focus_chain (GtkContainer  *container)
       
       container->has_focus_chain = FALSE;
       
-      g_object_set_data (G_OBJECT (container), "gtk-container-focus-chain",
+      g_object_set_data (G_OBJECT (container), 
+                         I_("gtk-container-focus-chain"),
                          NULL);
 
       tmp_list = chain;
@@ -2211,17 +2204,18 @@ gtk_container_unset_focus_chain (GtkContainer  *container)
 /**
  * gtk_container_set_focus_vadjustment:
  * @container: a #GtkContainer
- * @adjustment: an adjustment which should be adjusted when the focus is moved among the
- *   descendents of @container
+ * @adjustment: an adjustment which should be adjusted when the focus 
+ *   is moved among the descendents of @container
  * 
- * Hooks up an adjustment to focus handling in a container, so when a child of the 
- * container is focused, the adjustment is scrolled to show that widget. This function
- * sets the vertical alignment. See gtk_scrolled_window_get_vadjustment() for a typical
- * way of obtaining the adjustment and gtk_container_set_focus_hadjustment() for setting
+ * Hooks up an adjustment to focus handling in a container, so when a 
+ * child of the container is focused, the adjustment is scrolled to 
+ * show that widget. This function sets the vertical alignment. See 
+ * gtk_scrolled_window_get_vadjustment() for a typical way of obtaining 
+ * the adjustment and gtk_container_set_focus_hadjustment() for setting
  * the horizontal adjustment.
  *
- * The adjustments have to be in pixel units and in the same coordinate system as the 
- * allocation for immediate children of the container. 
+ * The adjustments have to be in pixel units and in the same coordinate 
+ * system as the allocation for immediate children of the container. 
  */
 void
 gtk_container_set_focus_vadjustment (GtkContainer  *container,
@@ -2245,7 +2239,7 @@ gtk_container_set_focus_vadjustment (GtkContainer  *container,
  * @container: a #GtkContainer
  *
  * Retrieves the vertical focus adjustment for the container. See
- * gtk_container_set_focus_vadjustment ().
+ * gtk_container_set_focus_vadjustment().
  *
  * Return value: the vertical focus adjustment, or %NULL if
  *   none has been set.
@@ -2265,17 +2259,18 @@ gtk_container_get_focus_vadjustment (GtkContainer *container)
 /**
  * gtk_container_set_focus_hadjustment:
  * @container: a #GtkContainer
- * @adjustment: an adjustment which should be adjusted when the focus is moved among the
- *   descendents of @container
+ * @adjustment: an adjustment which should be adjusted when the focus is 
+ *   moved among the descendents of @container
  * 
- * Hooks up an adjustment to focus handling in a container, so when a child of the 
- * container is focused, the adjustment is scrolled to show that widget. This function
- * sets the horizontal alignment. See gtk_scrolled_window_get_hadjustment() for a typical
- * way of obtaining the adjustment and gtk_container_set_focus_vadjustment() for setting
+ * Hooks up an adjustment to focus handling in a container, so when a child 
+ * of the container is focused, the adjustment is scrolled to show that 
+ * widget. This function sets the horizontal alignment. See gt
+ * k_scrolled_window_get_hadjustment() for a typical way of obtaining the 
+ * adjustment and gtk_container_set_focus_vadjustment() for setting
  * the vertical adjustment.
  *
- * The adjustments have to be in pixel units and in the same coordinate system as the 
- * allocation for immediate children of the container. 
+ * The adjustments have to be in pixel units and in the same coordinate 
+ * system as the allocation for immediate children of the container. 
  */
 void
 gtk_container_set_focus_hadjustment (GtkContainer  *container,
@@ -2433,7 +2428,7 @@ gtk_container_unmap (GtkWidget *widget)
  * the event's area with the child area, and sending the event.
  * 
  * In most cases, a container can simply either simply inherit the
- * ::expose implementation from #GtkContainer, or, do some drawing 
+ * #GtkWidget::expose implementation from #GtkContainer, or, do some drawing 
  * and then chain to the ::expose implementation from #GtkContainer.
  **/
 void
