@@ -65,6 +65,7 @@
 #include "gtksettings.h"
 #include "gtkwidget.h"
 #include "gtkwindow.h"
+#include "gtktooltip.h"
 #include "gtkprivate.h"
 #include "gtkdebug.h"
 #include "gtkalias.h"
@@ -135,6 +136,7 @@ const guint gtk_binary_age = GTK_BINARY_AGE;
 const guint gtk_interface_age = GTK_INTERFACE_AGE;
 
 static guint gtk_main_loop_level = 0;
+static gint pre_initialized = FALSE;
 static gint gtk_initialized = FALSE;
 static GList *current_events = NULL;
 
@@ -167,8 +169,8 @@ static const GDebugKey gtk_debug_keys[] = {
 /**
  * gtk_check_version:
  * @required_major: the required major version.
- * @required_minor: the required major version.
- * @required_micro: the required major version.
+ * @required_minor: the required minor version.
+ * @required_micro: the required micro version.
  * 
  * Checks that the GTK+ library in use is compatible with the
  * given version. Generally you would pass in the constants
@@ -349,7 +351,7 @@ static gboolean do_setlocale = TRUE;
 void
 gtk_disable_setlocale (void)
 {
-  if (gtk_initialized)
+  if (pre_initialized)
     g_warning ("gtk_disable_setlocale() must be called before gtk_init()");
     
   do_setlocale = FALSE;
@@ -417,6 +419,95 @@ static const GOptionEntry gtk_args[] = {
   { NULL }
 };
 
+#ifdef G_OS_WIN32
+
+static char *iso639_to_check = NULL;
+static char *iso3166_to_check = NULL;
+static char *script_to_check = NULL;
+static gboolean setlocale_called = FALSE;
+
+static BOOL CALLBACK
+enum_locale_proc (LPTSTR locale)
+{
+  LCID lcid;
+  char iso639[10];
+  char iso3166[10];
+  char *endptr;
+
+
+  lcid = strtoul (locale, &endptr, 16);
+  if (*endptr == '\0' &&
+      GetLocaleInfo (lcid, LOCALE_SISO639LANGNAME, iso639, sizeof (iso639)) &&
+      GetLocaleInfo (lcid, LOCALE_SISO3166CTRYNAME, iso3166, sizeof (iso3166)))
+    {
+      if (strcmp (iso639, iso639_to_check) == 0 &&
+	  ((iso3166_to_check != NULL &&
+	    strcmp (iso3166, iso3166_to_check) == 0) ||
+	   (iso3166_to_check == NULL &&
+	    SUBLANGID (LANGIDFROMLCID (lcid)) == SUBLANG_DEFAULT)))
+	{
+	  char language[100], country[100];
+	  char locale[300];
+
+	  if (script_to_check != NULL)
+	    {
+	      /* If lcid is the "other" script for this language,
+	       * return TRUE, i.e. continue looking.
+	       */
+	      if (strcmp (script_to_check, "Latn") == 0)
+		{
+		  switch (LANGIDFROMLCID (lcid))
+		    {
+		    case MAKELANGID (LANG_AZERI, SUBLANG_AZERI_CYRILLIC):
+		      return TRUE;
+		    case MAKELANGID (LANG_UZBEK, SUBLANG_UZBEK_CYRILLIC):
+		      return TRUE;
+		    case MAKELANGID (LANG_SERBIAN, SUBLANG_SERBIAN_CYRILLIC):
+		      return TRUE;
+		    case MAKELANGID (LANG_SERBIAN, 0x07):
+		      /* Serbian in Bosnia and Herzegovina, Cyrillic */
+		      return TRUE;
+		    }
+		}
+	      else if (strcmp (script_to_check, "Cyrl") == 0)
+		{
+		  switch (LANGIDFROMLCID (lcid))
+		    {
+		    case MAKELANGID (LANG_AZERI, SUBLANG_AZERI_LATIN):
+		      return TRUE;
+		    case MAKELANGID (LANG_UZBEK, SUBLANG_UZBEK_LATIN):
+		      return TRUE;
+		    case MAKELANGID (LANG_SERBIAN, SUBLANG_SERBIAN_LATIN):
+		      return TRUE;
+		    case MAKELANGID (LANG_SERBIAN, 0x06):
+		      /* Serbian in Bosnia and Herzegovina, Latin */
+		      return TRUE;
+		    }
+		}
+	    }
+
+	  SetThreadLocale (lcid);
+
+	  if (GetLocaleInfo (lcid, LOCALE_SENGLANGUAGE, language, sizeof (language)) &&
+	      GetLocaleInfo (lcid, LOCALE_SENGCOUNTRY, country, sizeof (country)))
+	    {
+	      strcpy (locale, language);
+	      strcat (locale, "_");
+	      strcat (locale, country);
+
+	      if (setlocale (LC_ALL, locale) != NULL)
+		setlocale_called = TRUE;
+	    }
+
+	  return FALSE;
+	}
+    }
+
+  return TRUE;
+}
+  
+#endif
+
 static void
 do_pre_parse_initialization (int    *argc,
 			     char ***argv)
@@ -430,10 +521,73 @@ do_pre_parse_initialization (int    *argc,
   g_set_print_handler (gtk_print);
 #endif
 
+  if (pre_initialized)
+    return;
+
+  pre_initialized = TRUE;
+
   if (do_setlocale)
     {
+#ifdef G_OS_WIN32
+      /* If some of the POSIXish environment variables are set, set
+       * the Win32 thread locale correspondingly.
+       */ 
+      char *p = getenv ("LC_ALL");
+      if (p == NULL)
+	p = getenv ("LANG");
+
+      if (p != NULL)
+	{
+	  p = g_strdup (p);
+	  if (strcmp (p, "C") == 0)
+	    SetThreadLocale (LOCALE_SYSTEM_DEFAULT);
+	  else
+	    {
+	      /* Check if one of the supported locales match the
+	       * environment variable. If so, use that locale.
+	       */
+	      iso639_to_check = p;
+	      iso3166_to_check = strchr (iso639_to_check, '_');
+	      if (iso3166_to_check != NULL)
+		{
+		  *iso3166_to_check++ = '\0';
+
+		  script_to_check = strchr (iso3166_to_check, '@');
+		  if (script_to_check != NULL)
+		    *script_to_check++ = '\0';
+
+		  /* Handle special cases. */
+		  
+		  /* The standard code for Serbia and Montenegro was
+		   * "CS", but MSFT uses for some reason "SP". By now
+		   * (October 2006), SP has split into two, "RS" and
+		   * "ME", but don't bother trying to handle those
+		   * yet. Do handle the even older "YU", though.
+		   */
+		  if (strcmp (iso3166_to_check, "CS") == 0 ||
+		      strcmp (iso3166_to_check, "YU") == 0)
+		    iso3166_to_check = "SP";
+		}
+	      else
+		{
+		  script_to_check = strchr (iso639_to_check, '@');
+		  if (script_to_check != NULL)
+		    *script_to_check++ = '\0';
+		  /* LANG_SERBIAN == LANG_CROATIAN, recognize just "sr" */
+		  if (strcmp (iso639_to_check, "sr") == 0)
+		    iso3166_to_check = "SP";
+		}
+
+	      EnumSystemLocales (enum_locale_proc, LCID_SUPPORTED);
+	    }
+	  g_free (p);
+	}
+      if (!setlocale_called)
+	setlocale (LC_ALL, "");
+#else
       if (!setlocale (LC_ALL, ""))
 	g_warning ("Locale not supported by C library.\n\tUsing the fallback 'C' locale.");
+#endif
     }
 
   gdk_pre_parse_libgtk_only ();
@@ -942,8 +1096,10 @@ _gtk_get_lc_ctype (void)
  * effect. (Note that this can change over the life of an
  * application.)  The default language is derived from the current
  * locale. It determines, for example, whether GTK+ uses the
- * right-to-left or left-to-right text direction. See
- * _gtk_get_lc_ctype() for notes on behaviour on Windows.
+ * right-to-left or left-to-right text direction.
+ *
+ * This function is equivalent to pango_language_get_default().  See
+ * that function for details.
  * 
  * Return value: the default language as a #PangoLanguage, must not be
  * freed
@@ -951,22 +1107,7 @@ _gtk_get_lc_ctype (void)
 PangoLanguage *
 gtk_get_default_language (void)
 {
-  gchar *lang;
-  PangoLanguage *result;
-  gchar *p;
-  
-  lang = _gtk_get_lc_ctype ();
-  p = strchr (lang, '.');
-  if (p)
-    *p = '\0';
-  p = strchr (lang, '@');
-  if (p)
-    *p = '\0';
-
-  result = pango_language_from_string (lang);
-  g_free (lang);
-  
-  return result;
+  return pango_language_get_default ();
 }
 
 void
@@ -1459,6 +1600,20 @@ gtk_main_do_event (GdkEvent *event)
       g_assert_not_reached ();
       break;
     }
+
+  if (event->type == GDK_ENTER_NOTIFY
+      || event->type == GDK_LEAVE_NOTIFY
+      || event->type == GDK_BUTTON_PRESS
+      || event->type == GDK_2BUTTON_PRESS
+      || event->type == GDK_3BUTTON_PRESS
+      || event->type == GDK_KEY_PRESS
+      || event->type == GDK_DRAG_ENTER
+      || event->type == GDK_GRAB_BROKEN
+      || event->type == GDK_MOTION_NOTIFY
+      || event->type == GDK_SCROLL)
+    {
+      _gtk_tooltip_handle_event (event);
+    }
   
   tmp_list = current_events;
   current_events = g_list_remove_link (current_events, tmp_list);
