@@ -253,6 +253,16 @@ queue_resize_on_group (GtkSizeGroup *size_group)
     queue_resize_on_widget (size_group->widgets->data, TRUE);
 }
 
+static void
+initialize_size_group_quarks (void)
+{
+  if (!size_groups_quark)
+    {
+      size_groups_quark = g_quark_from_static_string (size_groups_tag);
+      visited_quark = g_quark_from_string (visited_tag);
+    }
+}
+
 static void
 gtk_size_group_class_init (GtkSizeGroupClass *klass)
 {
@@ -273,7 +283,7 @@ gtk_size_group_class_init (GtkSizeGroupClass *klass)
   /**
    * GtkSizeGroup:ignore-hidden:
    *
-   * If %TRUE, hidden widgets are ignored when determining 
+   * If %TRUE, unmapped widgets are ignored when determining 
    * the size of the group.
    *
    * Since: 2.8
@@ -282,13 +292,12 @@ gtk_size_group_class_init (GtkSizeGroupClass *klass)
 				   PROP_IGNORE_HIDDEN,
 				   g_param_spec_boolean ("ignore-hidden",
 							 P_("Ignore hidden"),
-							 P_("If TRUE, hidden widgets are ignored "
+							 P_("If TRUE, unmapped widgets are ignored "
 							    "when determining the size of the group"),
 							 FALSE,
 							 GTK_PARAM_READWRITE));
-
-  size_groups_quark = g_quark_from_static_string (size_groups_tag);
-  visited_quark = g_quark_from_string (visited_tag);
+  
+  initialize_size_group_quarks ();
 }
 
 static void
@@ -356,7 +365,7 @@ gtk_size_group_get_property (GObject      *object,
  * Return value: a newly created #GtkSizeGroup
  **/
 GtkSizeGroup *
-gtk_size_group_new (GtkSizeGroupMode  mode)
+gtk_size_group_new (GtkSizeGroupMode mode)
 {
   GtkSizeGroup *size_group = g_object_new (GTK_TYPE_SIZE_GROUP, NULL);
 
@@ -414,10 +423,10 @@ gtk_size_group_get_mode (GtkSizeGroup *size_group)
 /**
  * gtk_size_group_set_ignore_hidden:
  * @size_group: a #GtkSizeGroup
- * @ignore_hidden: whether hidden widgets should be ignored
+ * @ignore_hidden: whether unmapped widgets should be ignored
  *   when calculating the size
  * 
- * Sets whether invisible widgets should be ignored when
+ * Sets whether unmapped widgets should be ignored when
  * calculating the size.
  *
  * Since: 2.8 
@@ -473,7 +482,10 @@ gtk_size_group_widget_destroyed (GtkWidget    *widget,
  * and the requisition of the other widgets in the size group.
  * Whether this applies horizontally, vertically, or in both directions
  * depends on the mode of the size group. See gtk_size_group_set_mode().
- **/
+ *
+ * When the widget is destroyed or no longer referenced elsewhere, it will 
+ * be removed from the size group.
+ */
 void
 gtk_size_group_add_widget (GtkSizeGroup     *size_group,
 			   GtkWidget        *widget)
@@ -510,8 +522,8 @@ gtk_size_group_add_widget (GtkSizeGroup     *size_group,
  * Removes a widget from a #GtkSizeGroup.
  **/
 void
-gtk_size_group_remove_widget (GtkSizeGroup     *size_group,
-			      GtkWidget        *widget)
+gtk_size_group_remove_widget (GtkSizeGroup *size_group,
+			      GtkWidget    *widget)
 {
   GSList *groups;
   
@@ -548,7 +560,7 @@ gtk_size_group_remove_widget (GtkSizeGroup     *size_group,
 GSList *
 gtk_size_group_get_widgets (GtkSizeGroup *size_group)
 {
-     return size_group->widgets;
+  return size_group->widgets;
 }
 
 static gint
@@ -632,7 +644,7 @@ compute_dimension (GtkWidget        *widget,
 
 	      gint dimension = compute_base_dimension (tmp_widget, mode);
 
-	      if (GTK_WIDGET_VISIBLE (tmp_widget) || !group->ignore_hidden)
+	      if (GTK_WIDGET_MAPPED (tmp_widget) || !group->ignore_hidden)
 		{
 		  if (dimension > result)
 		    result = dimension;
@@ -732,6 +744,8 @@ void
 _gtk_size_group_get_child_requisition (GtkWidget      *widget,
 				       GtkRequisition *requisition)
 {
+  initialize_size_group_quarks ();
+
   if (requisition)
     {
       if (get_size_groups (widget))
@@ -761,6 +775,8 @@ _gtk_size_group_compute_requisition (GtkWidget      *widget,
   gint width;
   gint height;
 
+  initialize_size_group_quarks ();
+
   if (get_size_groups (widget))
     {
       /* Only do the full computation if we actually have size groups */
@@ -792,6 +808,8 @@ _gtk_size_group_compute_requisition (GtkWidget      *widget,
 void
 _gtk_size_group_queue_resize (GtkWidget *widget)
 {
+  initialize_size_group_quarks ();
+
   queue_resize_on_widget (widget, TRUE);
 }
 
