@@ -183,24 +183,6 @@ gtk_stock_lookup (const gchar  *stock_id,
   return found != NULL;
 }
 
-static void
-listify_foreach (gpointer key, gpointer value, gpointer data)
-{
-  GSList **list = data;
-
-  *list = g_slist_prepend (*list, key);
-}
-
-static GSList *
-g_hash_table_get_keys (GHashTable *table)
-{
-  GSList *list = NULL;
-
-  g_hash_table_foreach (table, listify_foreach, &list);
-
-  return list;
-}
-
 /**
  * gtk_stock_list_ids:
  * 
@@ -213,42 +195,40 @@ g_hash_table_get_keys (GHashTable *table)
 GSList*
 gtk_stock_list_ids (void)
 {
-  GSList *ids;
-  GSList *icon_ids;
+  GList *ids;
+  GList *icon_ids;
   GSList *retval;
-  GSList *tmp_list;
   const gchar *last_id;
   
   init_stock_hash ();
 
   ids = g_hash_table_get_keys (stock_hash);
   icon_ids = _gtk_icon_factory_list_ids ();
-  ids = g_slist_concat (ids, icon_ids);
+  ids = g_list_concat (ids, icon_ids);
 
-  ids = g_slist_sort (ids, (GCompareFunc)strcmp);
+  ids = g_list_sort (ids, (GCompareFunc)strcmp);
 
   last_id = NULL;
   retval = NULL;
-  tmp_list = ids;
-  while (tmp_list != NULL)
+  while (ids != NULL)
     {
-      GSList *next;
+      GList *next;
 
-      next = g_slist_next (tmp_list);
+      next = g_list_next (ids);
 
-      if (last_id && strcmp (tmp_list->data, last_id) == 0)
+      if (last_id && strcmp (ids->data, last_id) == 0)
         {
           /* duplicate, ignore */
         }
       else
         {
-          retval = g_slist_prepend (retval, g_strdup (tmp_list->data));
-          last_id = tmp_list->data;
+          retval = g_slist_prepend (retval, g_strdup (ids->data));
+          last_id = ids->data;
         }
 
-      g_slist_free_1 (tmp_list);
+      g_list_free_1 (ids);
       
-      tmp_list = next;
+      ids = next;
     }
 
   return retval;
@@ -321,10 +301,12 @@ static const GtkStockItem builtin_items [] =
   { GTK_STOCK_CDROM, N_("_CD-Rom"), 0, 0, GETTEXT_PACKAGE },
   { GTK_STOCK_CLEAR, N_("_Clear"), 0, 0, GETTEXT_PACKAGE },
   { GTK_STOCK_CLOSE, N_("_Close"), GDK_CONTROL_MASK, 'w', GETTEXT_PACKAGE },
+  { GTK_STOCK_CONNECT, N_("C_onnect"), 0, 0, GETTEXT_PACKAGE },
   { GTK_STOCK_CONVERT, N_("_Convert"), 0, 0, GETTEXT_PACKAGE },
-  { GTK_STOCK_COPY, N_("_Copy"), GDK_CONTROL_MASK, 'c', GETTEXT_PACKAGE },
+   { GTK_STOCK_COPY, N_("_Copy"), GDK_CONTROL_MASK, 'c', GETTEXT_PACKAGE },
   { GTK_STOCK_CUT, N_("Cu_t"), GDK_CONTROL_MASK, 'x', GETTEXT_PACKAGE },
   { GTK_STOCK_DELETE, N_("_Delete"), 0, 0, GETTEXT_PACKAGE },
+  { GTK_STOCK_DISCONNECT, N_("_Disconnect"), 0, 0, GETTEXT_PACKAGE },
   { GTK_STOCK_EXECUTE, N_("_Execute"), 0, 0, GETTEXT_PACKAGE },
   { GTK_STOCK_EDIT, N_("_Edit"), 0, 0, GETTEXT_PACKAGE },
   { GTK_STOCK_FIND, N_("_Find"), GDK_CONTROL_MASK, 'f', GETTEXT_PACKAGE },
@@ -387,6 +369,10 @@ static const GtkStockItem builtin_items [] =
   { GTK_STOCK_NO, N_("_No"), 0, 0, GETTEXT_PACKAGE },
   { GTK_STOCK_OK, N_("_OK"), 0, 0, GETTEXT_PACKAGE },
   { GTK_STOCK_OPEN, N_("_Open"), GDK_CONTROL_MASK, 'o', GETTEXT_PACKAGE },
+  { GTK_STOCK_ORIENTATION_LANDSCAPE, N_("Landscape"), 0, 0, GETTEXT_PACKAGE },
+  { GTK_STOCK_ORIENTATION_PORTRAIT, N_("Portrait"), 0, 0, GETTEXT_PACKAGE },
+  { GTK_STOCK_ORIENTATION_REVERSE_LANDSCAPE, N_("Reverse landscape"), 0, 0, GETTEXT_PACKAGE },
+  { GTK_STOCK_ORIENTATION_REVERSE_PORTRAIT, N_("Reverse portrait"), 0, 0, GETTEXT_PACKAGE },
   { GTK_STOCK_PASTE, N_("_Paste"), GDK_CONTROL_MASK, 'v', GETTEXT_PACKAGE },
   { GTK_STOCK_PREFERENCES, N_("_Preferences"), 0, 0, GETTEXT_PACKAGE },
   { GTK_STOCK_PRINT, N_("_Print"), 0, 0, GETTEXT_PACKAGE },
@@ -399,6 +385,7 @@ static const GtkStockItem builtin_items [] =
   { GTK_STOCK_REVERT_TO_SAVED, N_("_Revert"), 0, 0, GETTEXT_PACKAGE },
   { GTK_STOCK_SAVE, N_("_Save"), GDK_CONTROL_MASK, 's', GETTEXT_PACKAGE },
   { GTK_STOCK_SAVE_AS, N_("Save _As"), 0, 0, GETTEXT_PACKAGE },
+  { GTK_STOCK_SELECT_ALL, N_("Select _All"), 0, 0, GETTEXT_PACKAGE },
   { GTK_STOCK_SELECT_COLOR, N_("_Color"), 0, 0, GETTEXT_PACKAGE },
   { GTK_STOCK_SELECT_FONT, N_("_Font"), 0, 0, GETTEXT_PACKAGE },
   { GTK_STOCK_SORT_ASCENDING, N_("_Ascending"), 0, 0, GETTEXT_PACKAGE },
