@@ -350,9 +350,6 @@ gtk_tooltip_reset (GtkTooltip *tooltip)
 static gboolean
 gtk_tooltip_paint_window (GtkTooltip *tooltip)
 {
-  GtkRequisition req;
-
-  gtk_widget_size_request (tooltip->window, &req);
   gtk_paint_flat_box (tooltip->window->style,
 		      tooltip->window->window,
 		      GTK_STATE_NORMAL,
@@ -634,13 +631,13 @@ gtk_tooltip_show_tooltip (GdkDisplay *display)
     {
       window = tooltip->last_window;
 
+      if (!GDK_IS_WINDOW (window))
+	return;
+
       gdk_window_get_origin (window, &x, &y);
       x = tooltip->last_x - x;
       y = tooltip->last_y - y;
 
-      if (!window)
-	return;
-
       pointer_widget = tooltip_widget = find_widget_under_pointer (window,
 								   &x, &y);
     }
