@@ -329,7 +329,7 @@ gtk_tooltips_set_tip (GtkTooltips *tooltips,
 
       tooltips->tips_data_list = g_list_append (tooltips->tips_data_list,
                                                 tooltipsdata);
-      g_signal_connect_after (widget, "event-after",
+      g_signal_connect_after (widget, "event_after",
                               G_CALLBACK (gtk_tooltips_event_handler),
 			      tooltipsdata);
 
@@ -438,29 +438,25 @@ gtk_tooltips_draw_tips (GtkTooltips *tooltips)
   else
     y = y + widget->allocation.height + 4;
 
-  toplevel = gtk_widget_get_toplevel (widget);
+  toplevel = GTK_WINDOW (gtk_widget_get_toplevel (widget));
   if (toplevel && GTK_IS_WINDOW (toplevel))
-    gtk_window_set_transient_for (tooltips->tip_window, toplevel);
+    gtk_window_set_transient_for (GTK_WINDOW (tooltips->tip_window), toplevel);
   
   gtk_window_move (GTK_WINDOW (tooltips->tip_window), x, y);
   gtk_widget_show (tooltips->tip_window);
 }
 
-static gint
+static gboolean
 gtk_tooltips_timeout (gpointer data)
 {
   GtkTooltips *tooltips = (GtkTooltips *) data;
 
-  GDK_THREADS_ENTER ();
-  
   if (tooltips->active_tips_data != NULL &&
       GTK_WIDGET_DRAWABLE (tooltips->active_tips_data->widget))
     gtk_tooltips_draw_tips (tooltips);
 
   tooltips->timer_tag = 0;
 
-  GDK_THREADS_LEAVE ();
-
   return FALSE;
 }
 
@@ -620,7 +616,7 @@ gtk_tooltips_start_delay (GtkTooltips *tooltips,
 	delay = STICKY_DELAY;
       else
 	delay = tooltips->delay;
-      tooltips->timer_tag = g_timeout_add (delay,
+      tooltips->timer_tag = gdk_threads_add_timeout (delay,
 					   gtk_tooltips_timeout,
 					   (gpointer) tooltips);
     }
@@ -783,7 +779,7 @@ gtk_tooltips_get_info_from_tip_window (GtkWindow    *tip_window,
   if (tooltips)
     *tooltips = current_tooltips;
   if (current_widget)
-    *current_widget = has_tips ? current_tooltips->active_tips_data->widget : NULL;
+    *current_widget = (has_tips && current_tooltips->active_tips_data) ? current_tooltips->active_tips_data->widget : NULL;
 
   return has_tips;
 }
